@@ -17,10 +17,14 @@ import ujson
 import os
 import shutil
 
+from typing import List
+
 from clade.utils import get_logger, merge_preset_to_conf
 from clade.intercept import intercept
 from clade.extensions.abstract import Extension
 from clade.types.nested_dict import nested_dict, traverse
+from clade.cmds import iter_cmds, iter_cmds_by_which
+from clade.envs import iter_envs
 
 
 class Clade:
@@ -76,7 +80,7 @@ class Clade:
         conf = merge_preset_to_conf(preset, conf)
 
         conf["work_dir"] = self.work_dir
-        conf["log_level"] = "INFO"
+        conf["log_level"] = conf.get("log_level", "INFO")
         conf["cmds_file"] = self.cmds_file
         conf["preset"] = preset
         # conf["Info.cif"] = args.cif if args.cif else conf.get("Info.cif", "cif")
@@ -139,7 +143,7 @@ class Clade:
 
         self.__dump_conf()
 
-    def intercept(self, command, cwd=os.getcwd(), append=False, use_wrappers=False, intercept_open=False):
+    def intercept(self, command, cwd=os.getcwd(), append=False, use_wrappers=False, intercept_open=False, intercept_envs=False):
         """Execute intercepting of build commands.
 
         Args:
@@ -163,6 +167,7 @@ class Clade:
             append=append,
             use_wrappers=use_wrappers,
             intercept_open=intercept_open,
+            intercept_envs=intercept_envs,
             conf=self.conf
         )
 
@@ -336,6 +341,45 @@ class Clade:
         """List of all parsed commands."""
         return self.CmdGraph.load_all_cmds()
 
+    def get_raw_cmds(self):
+        """Get an iterator over all unparsed commands."""
+        for cmd in iter_cmds(self.cmds_file):
+            yield cmd
+
+    def get_raw_cmds_by_which(self, which_list: List[str]):
+        """Get an iterator over all unparsed commands filtered by 'which' field."""
+        for cmd in iter_cmds_by_which(self.cmds_file, which_list):
+            yield cmd
+
+    def get_raw_cmd_by_id(self, cmd_id: str):
+        """Get raw command by its identifier."""
+        for cmd in iter_cmds(self.cmds_file):
+            if cmd["id"] == cmd_id:
+                return cmd
+
+        return RuntimeError("No command with id {}".format(cmd_id))
+
+    def get_envs(self):
+        """Get an iterator over all environment variables."""
+        for env in iter_envs(os.path.join(self.work_dir, "envs.txt")):
+            yield env
+
+    def get_envs_by_id(self, cmd_id: str):
+        """Get environment variables by its intercepted command identifier."""
+        for envs in iter_envs(os.path.join(self.work_dir, "envs.txt")):
+            if envs["id"] == cmd_id:
+                return envs["envs"]
+
+        raise RuntimeError("No envs with id {}".format(cmd_id))
+
+    def get_env_value_by_id(self, cmd_id: str, name: str):
+        """Get environment variable by its intercepted command identifier and name."""
+        envs = self.get_envs_by_id(cmd_id)
+        if name in envs:
+            return envs[name]
+        else:
+            raise RuntimeError("No envs with id {} and name {}".format(cmd_id, name))
+
     def get_cmds(self, with_opts=False, with_raw=False):
         """Get list with all parsed commands."""
         return self.CmdGraph.load_all_cmds(with_opts=with_opts, with_raw=with_raw)
@@ -360,7 +404,7 @@ class Clade:
         if not cmd_type:
             cmd_type = self.get_cmd_type(cmd_id)
 
-        if with_deps and cmd_type not in ["CC", "CL"]:
+        if with_deps and cmd_type not in ["CC", "CL", "CXX"]:
             raise RuntimeError("Only compiler commands have dependencies")
 
         ext_obj = self.CmdGraph.get_ext_obj(cmd_type)
@@ -400,7 +444,7 @@ class Clade:
         if not cmd_type:
             cmd_type = self.get_cmd_type(cmd_id)
 
-        if cmd_type not in ["CC", "CL"]:
+        if cmd_type not in ["CC", "CL", "CXX"]:
             raise RuntimeError("Only compiler commands have dependencies")
 
         cc_obj = self.CmdGraph.get_ext_obj(cmd_type)
