@@ -84,9 +84,7 @@ class CC(Compiler):
             ))
             deps_file = self.__collect_deps(cmd_id, which, cmd, cmd_in)
 
-            # Remove duplicates
-            for dep in [d for d in self.__parse_deps(deps_file) if d not in deps]:
-                deps.append(dep)
+            deps.extend(self.__parse_deps(deps_file))
 
         return deps
 
@@ -98,6 +96,9 @@ class CC(Compiler):
         else:
             additional_opts = ["-Wp,-MMD,{}".format(deps_file), "-MM"]
 
+        if not os.path.isabs(which):
+            which = os.path.join(cmd["cwd"], which)
+
         opts = cmd["opts"] + additional_opts
         command = [which] + opts + [cmd_in]
 
